import 'dart:io' as io;

import 'package:flex/flex.dart';
import 'package:plexus/plexus.dart';
import 'package:quiver/strings.dart' as StringQ;

/// A handler for Plexus modules that formats and forwards all [event] to [out]
///
/// [out] defaults to [io.stdout]
void consoleHandler(event, [io.Stdout out]) {
  out ??= io.stdout;
  if (event.level >= event.source.level) {
    final timeFmt = AnsiCode.diamond.wrap(iso8601NoTZ.format(event.timestamp));
    final tag = event.level.wrappedName;
    final loggerName = AnsiCode.lightMagenta.wrap(event.source.fullName);
    String msg;
    if (event.level == Level.PRINT) {
      msg = event.message;

      // TODO(mpfaff): add `!` non-null assertion to [longMessage]
      if (event.longMessage != null && event.longMessage.isNotEmpty) {
        msg += '\n | ';
        // TODO(mpfaff): add `!` non-null assertion to [longMessage]
        final lines = event.longMessage.split('\n');
        while (lines.last.isEmpty) {
          lines.removeLast();
        }
        lines.forEach((l) => msg += '\n | $l');
        msg += '\n | ';
      }

      msg = event.level.ansiCode.wrap(msg);
    } else if (event.level == Level.ERROR || event.level == Level.FATAL) {
      //`- 3` is for padding
      final len = io.stdout.hasTerminal ? io.stdout.terminalColumns - 3 : 56;
      // final errorMsg = '―――――――――――――― ${event.error ?? 'Auto-generated stacktrace'} ――――――――――――――';
      final errorMsg = StringQ.center('Auto-generated stacktrace', len, '―');
      msg = '${event.message}\n |$errorMsg|';
      final stackList = (event.stackTrace ?? StackTrace.current).toString().split('\n');

      while (stackList.last.isEmpty) {
        stackList.removeLast();
      }

      stackList.forEach((l) => msg += '\n | $l');

      msg = '$msg\n |${'―' * len}|';

      msg = event.level.ansiCode.wrap(msg);

      msg = '[$timeFmt]: [$tag]: [$loggerName]: ' + msg;
    } else {
      msg = event.message;

      // TODO(mpfaff): add `!` non-null assertion to [longMessage]
      if (event.longMessage != null && event.longMessage.isNotEmpty) {
        msg += '\n | ';
        // TODO(mpfaff): add `!` non-null assertion to [longMessage]
        final lines = event.longMessage.split('\n');
        while (lines.last.isEmpty) {
          lines.removeLast();
        }
        lines.forEach((l) => msg += '\n | $l');
        msg += '\n | ';
      }

      msg = event.level.ansiCode.wrap(msg);

      msg = '[$timeFmt]: [$tag]: [$loggerName]: ' + msg;
    }

    out.writeln(msg);
  }
}
