import 'dart:io';

import 'package:flex/flex.dart';
import 'package:mockito/mockito.dart';
import 'package:plexus/plexus.dart';
import 'package:test/test.dart';

void main() {
  group('root_module', () {
    test('root_module_init', () async {
      await RootModule().start(() {
        expect(RootModule().state, State.initialized, reason: 'After running RootModule().start(), the state should be initialized.');
      });
    });

    test('root_module_inheritance', () {
      expect(RootModule() is RootModule, true, reason: 'RootModule() should be of type RootModule.');
      expect(RootModule() is Module, true, reason: 'RootModule() should inherit from Module.');
      expect(RootModule() is ModuleCommon, true, reason: 'RootModule() should inherit from ModuleCommon.');
    });

    test('root_module_identity', () {
      expect(RootModule() == RootModule(), true, reason: 'RootModule should be equal to RootModule.');
      expect(identical(RootModule(), RootModule()), true, reason: 'RootModule should be identical to RootModule (must be the same instance).');
    });

    test('root_module_level', () {
      final root = RootModule();
      expect(root.level == RootModule.defaultLevel, true, reason: 'On a RootModule with default settings, `level` should be `defaultLevel` (${RootModule.defaultLevel}).');
    });

    test('root_module_log', () async {
      final fake = MockStdout();
      final root = RootModule(onDispose: () {
        expect(fake.buffer[fake.buffer.length - 2].contains('Hello, World!'), true);
      });
      root.subscribe('__console__', (e) => consoleHandler(e, fake));
      await root.start(() async {
        root.log(Level.SUCCESS, 'Hello, World!');
      });
    });
  });
}

class MockStdout extends Mock implements Stdout {
  final List<dynamic> buffer = [];

  @override
  void writeln([dynamic object = '']) {
    buffer.add(object);
    stdout.writeln(object);
  }
}
